/**
  *This class is a way to graphically represent real world data.
  *It uses fundamental graph theory which includes the concept of
  *verticies, edges and edges with weights.
  */
import java.util.Vector;
import java.io.Serializable;
import queue.Queue;
import java.util.Stack;

public class Graph implements GraphInterface, Serializable
{

  /**
    *2-D matrix used for holding the location of edges
    */
  protected double[][] edges;
  /**
    *Vector containing Vertices.
    */
  protected Vector items;
  /**
    *true if graph is weighted.
    */
  protected final boolean DIRECTED;
  protected final boolean WEIGHTED;
  /**
    *the initial size of matrix.
    */
  private final int INITIAL_SIZE = 50;
  /**
    *the resize factor of matrix.
    */
  private final int RESIZE_FACTOR = 50;

  /**
    *creates a non-directed, non-weighted Graph.<br>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI><I>none</I>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>An empty unweighted graph will be created (i.e. each node has a weight of exactly 1)
    *</UL>
    */
   public Graph()
   {
      this(false); //sets the graph up as being undirected by using the other constructor.
   }
  /**
    *creates a weighted, directed Graph if true, non-directed, non-weighted if false.<br>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>A boolean must be passed to it indicating you want a weighted graph (i.e. <code>directed</code> must be true.
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>An empty weighted graph will be created.
    *</UL>
    */
   public Graph(boolean directed)
   {
      DIRECTED = directed; //sets the graph as being weighted
      
      //all this added in later when I wanted to add functionality.
      //the javadoc API for Graph() and Graph(boolean d) un untocuhed by
     //adding these few lines
      if(DIRECTED)
       { WEIGHTED = true;}
      else
       { WEIGHTED = false;}

      //initalize our class fields
      items = new Vector();
      edges = new double[INITIAL_SIZE][INITIAL_SIZE];
      fillEmpty(edges); //set everything to POSITIVE_INFINITY
   }
  /**
    *creates a graph.
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>You must specify if you want a weighted graph</LI>
    *  <LI>You must specify if you want a directed graph</LI>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>An empty graph meeting your specifications will be created
    *</UL>
    */
   public Graph(boolean weighted, boolean directed)
   {
     //filed with function arguments
     DIRECTED = directed;
     WEIGHTED = weighted;

     //standard init stuff
      items = new Vector();
      edges = new double[INITIAL_SIZE][INITIAL_SIZE];
      fillEmpty(edges); //set everything to POSITIVE_INFINITY
   }

   public void makeEmpty()
   {
     //same as found in constructor
      items = new Vector();
      edges = new double[INITIAL_SIZE][INITIAL_SIZE];
      fillEmpty(edges);
   }

   public boolean isEmpty()
   {
      return items.isEmpty(); //checks to see if the vector is empty and returns it.
   }

   public int numVertices()
   {
      return items.size(); //returns the sizee of the vector
                           //which is equal to the number of verticies
   }

   public int numEdges()
   {
      int count = 0; //our counting variable
      for(int x = 0; x < edges.length; x++)//select a row
      {
          for(int y = 0; y < edges.length; y++)//process each column in that row
          {
              //if that column is not POSITIVE_INFINITY, increment count
              if(edges[x][y] != Double.POSITIVE_INFINITY)
                { count++; }
          }
       }
      return count; //return count
   }

   public void addVertex(GraphNode myItem) throws GraphException
   {
     if(items.contains(myItem))  //test for duplicate
       { throw new GraphException("Duplicate Item"); } //throw exception if duplicate
     items.add(myItem); //add the item to the end of the vector
     if(items.size() > edges.length) //check to see if matrix is big enough to hold edges
       {
         edges = resize(edges, edges.length + RESIZE_FACTOR);
       } //resizes edges matrix by resize factor
   }

   public void addEdge(Comparable searchKey1, Comparable searchKey2) throws GraphException
   {
      int temp1 = items.indexOf(new GraphNode((String) searchKey1)); //get the index for key1
      int temp2 = items.indexOf(new GraphNode((String) searchKey2)); //get the index fo key 2

      if (temp1 == -1 || temp2 == -1) //check for non-existance of search keys
       { throw new GraphException("Search Key Invalid"); } //throw exception
      if (temp1 == temp2) //check for self loop
       {throw new GraphException("Self Loops not allowed"); } //self loop exception
      //key1 is row/source
      //key2 is column/destination
      if( DIRECTED ) //checks to see if it's directed
        { edges[temp1][temp2] = 1; }//give it a weight of 1 since none was specified
      else //now we have to update two sections of matrix with 1 since graph is undirected
        {
          edges[temp1][temp2] = 1;
          edges[temp2][temp1] = 1; //invert row-columns
        }
   }

   public void addEdge(Comparable searchKey1, Comparable searchKey2, double weight) throws GraphException
   {
     if(WEIGHTED == false) //checks to see if this constant was originally set to false
       { throw new GraphException("Originally constructed as unweighted. Weighted graph unusable"); } //give them error message

     //almost everything else is the same as the previous function
      int temp1 = items.indexOf(new GraphNode((String) searchKey1)); //get the index for key1
      int temp2 = items.indexOf(new GraphNode((String) searchKey2)); //get the index fo key 2

      if (temp1 == -1 || temp2 == -1) //check for non-existance of search keys
       { throw new GraphException("Search Key(s) Invalid"); } //throw exception

      if (temp1 == temp2) //check for self loop
       {throw new GraphException("Self Loops not allowed"); } //self loop exception

      //key1 is row/source
      //key2 is column/destination
      if( DIRECTED )
        { edges[temp1][temp2] = weight;} //DIFFERNT FROM ABOVE FUNCTION
      else
        {
          edges[temp1][temp2] = weight;
          edges[temp2][temp1] = weight;
        }
   }

   public double getWeight(Comparable searchKey1, Comparable searchKey2) throws GraphException
   {
     if(WEIGHTED == false) //checks to see if this constant was originally set to false
       { throw new GraphException("Originally constructed as undirected. Cannot getWeight()"); } //give them error message
     //almost everything else is the same as the previous function
      int temp1 = items.indexOf(new GraphNode((String) searchKey1)); //get the index for key1
      int temp2 = items.indexOf(new GraphNode((String) searchKey2)); //get the index fo key 2

      if (temp1 == -1 || temp2 == -1) //check for non-existance of search keys
       { throw new GraphException("Search Key(s) Invalid"); } //throw exception

      //key1 is row/source
      //key2 is column/destination
      return edges[temp1][temp2];
   }

   public void removeEdge(Comparable searchKey1, Comparable searchKey2) throws GraphException
   {
      int temp1 = items.indexOf(new GraphNode((String) searchKey1)); //get the index for key1
      int temp2 = items.indexOf(new GraphNode((String) searchKey2)); //get the index fo key 2

      if (temp1 == -1 || temp2 == -1) //check for non-existance of search keys
       { throw new GraphException("Search Key(s) Invalid"); } //throw exception
      if (temp1 == temp2) //check for self loop
       {throw new GraphException("Self Loops not allowed"); } //self loop exception

      //key1 is row/source
      //key2 is column/destination
      if(DIRECTED)
      { edges[temp1][temp2] = Double.POSITIVE_INFINITY; } //only need to remove one if directed
      else
      {
        edges[temp1][temp2] = Double.POSITIVE_INFINITY;
        edges[temp2][temp1] = Double.POSITIVE_INFINITY; //remove it and it's inverse if undirected
      }
   }

   public GraphNode removeVertex(Comparable searchKey) throws GraphException
   {
      int vdel = items.indexOf(new GraphNode((String) searchKey)); //check to see if node exists
      if (vdel == -1) //if it doesn't
        {throw new GraphException("Search Key Invalid"); } //throw exception

      //now lets take care of the matrix first
      delElement(edges, vdel);

      //now for the vector
      return (GraphNode) items.remove(vdel);//removes element from vector and returns it to user
   }

   public GraphNode getVertex(Comparable searchKey) throws GraphException
   {
      int itm = items.indexOf(new GraphNode((String) searchKey)); //get the index from vector
      if (itm == -1) //check to see if it exists
        {throw new GraphException("Search Key Invalid"); } //throw exception
      else
        { return (GraphNode) items.elementAt(itm); }
   }

   public GraphNode getVertex(int index) throws GraphException
   {
      if(index > items.size())
       { throw new GraphException("Index out of bounds: " + index); }
      else
       { return (GraphNode) items.elementAt(index); }
   }

   /**
     *stack used by recursive dfs method.
     */
   private Stack depthStack;

   /**
     *vector returned from dfs search.
     */
   private Vector depthVector;

   public Vector dfs(Comparable searchKey) throws GraphException
   {
      clearAllMarks();
      depthVector = new Vector();
      depthStack = new Stack(); //vector to return
      goDeep(getVertex(searchKey)); //call recursive function
 
      while(! depthStack.empty())
        { depthVector.add(depthStack.pop()); }

      return depthVector;
   }

   /**
    * Provides recursive method for traversing graph via dfs-technique.<br/>
    * <strong>Preconditions:</strong><br/>
    * <ul>
    *    <li>Takes Comparable-typed searchKey that represents which GraphNode to start with</li>
    * </ul>
    * <strong>Postconditions:</strong><br/>
    * <ul>
    *    <li>Returns Vector with GraphNodes in order of dfs traversal</li>
    * </ul>
    */
   private void goDeep(GraphNode node)
   {
   //loop through items vector
      for (int i = 0; i < items.size(); i++)
      {
        //if edge is not infinite and GraphNode is NOT marked
        if( (edges[items.indexOf(node)][i] < Double.POSITIVE_INFINITY) && (! getVertex(i).isMarked() )  )
        {
          node.mark();
          goDeep(getVertex(i));
        }

      }
     depthStack.push(node);
   }

   /**
    * Provides iterative method for traversing graph via bfs-technique.<br/>
    * <strong>Preconditions:</strong><br/>
    * <ul>
    *    <li>Takes Comparable-typed searchKey that represents which GraphNode to start with</li>
    * </ul>
    * <strong>Postconditions:</strong><br/>
    * <ul>
    *    <li>Returns Vector with GraphNodes in order of dfs traversal</li>
    * </ul>
    * <em>Algorithm based upon pseudo-code in "Walls & Mirrors", 
    * pages 622-623, and extended by Bryce.</em>
    */
   public Vector bfs(Comparable searchKey) throws GraphException
   {
      clearAllMarks();
      Vector breadthVector = new Vector();
         
      /* instantiate queue */
      //q.createQueue()
      Queue myQueue = new Queue();

      /*add vertex to queue and mark it */
      //q.enqueue(v)
      //Mark v as visited
      myQueue.enqueue(getVertex(searchKey));
      getVertex(searchKey).mark();
      
      
      //while (!q.isEmpty()){
      //w = q.dequeue()
      while (! myQueue.isEmpty())
      { 
         GraphNode w = new GraphNode( ((GraphNode) myQueue.dequeue()).toString() );
         breadthVector.add(w);

      /* loop invariant: there is a path from vertex w to every vertex in the queue q */
      //for (each unvisited vertex u adjacent to w) {
      //Mark u as visited
      //q.enqueue(u)
      //}}
         for (int i = 0; i < items.size(); i++)
         {
           if((edges[items.indexOf(w)][i] < Double.POSITIVE_INFINITY) && (! getVertex(i).isMarked()))
           {
             getVertex(i).mark();
             myQueue.enqueue(getVertex(i));
           }      
         }
      }
      /* added step */
      //unload queue into Vector for output
         while(! myQueue.isEmpty())
            { breadthVector.add(myQueue.dequeue()); }
         return breadthVector;
   }

/**
  *distance returned from shortest path.
  */
  private double distance = -1;
   /**
    * returns the distance of the shortest past from previous search.<br />
    * <strong>Preconditions:</strong>
    * <ul>
    *    <li>A shortest path search must have been preformed prior to this.</li>
    * </ul>
    * <strong>Postconditions:</strong>
    * <ul>
    *    <li>Will return the distance of the shortest path or -1 if function was not run prior to this.</li>
    * </ul>
    */
   public double getDistance()
   { return distance; }

   /**
    * Searches for the shortest path between two vertices.<br />
    * <strong>Preconditions:</strong>
    * <ul>
    *    <li>Takes two Comparable-typed searchKeys; one starting point, other ending point</li>
    * </ul>
    * <strong>Postconditions:</strong>
    * <ul>
    *    <li>Returns Vector representing shortest path</li>
    * </ul>
    * <em>Algorithm based upon pseudo-code in "Walls & Mirrors", 
    * pages 635-637, and extended by Bryce.</em>
    */
   public Vector shortestPath (Comparable searchKey1, Comparable searchKey2) throws GraphException
   {
      if( searchKey1.equals(searchKey2) )  //throw an exception for self loop
       { throw new GraphException("Self Loop Not Allowed"); }

      clearAllMarks(); //clear our marks

      getVertex(searchKey1).mark(); //mark the inital
      
      int n = this.numVertices(); //total number of elements needed for loops
           
      double[] weight = new double[n];                  //holds weight array
      double smallestWeight = Double.POSITIVE_INFINITY;
      GraphNode[] path = new GraphNode[n];              //holds path array
      Stack returned = new Stack();                     //used to compute what needs to be returned
      Vector retval  = new Vector();                    //what is actually returned

      for(int l = 0; l < n; l++) //fill up path array
        { path[l] = (GraphNode) items.elementAt(l); }
      for(int v = 0; v < n; v++) //fill up weights with INFINITY for first if condition
        { weight[v] = Double.POSITIVE_INFINITY; }
      int source = 0; //starting position
      int next = items.indexOf( new GraphNode( (String) searchKey1) ); //stating step
      weight[next] = 0;
         
      while(true) //ended with a break
      { 
         smallestWeight = Double.POSITIVE_INFINITY; //reset each time for first if statement
         source = next;                             //save current source for use for reset
         getVertex(source).mark();                  //mark the source on entry

         for (int v = 0; v < n; v++)
         {
              //controls the weight array and path array
              if (  !(getVertex(v).isMarked())  && (weight[v] > weight[source] + edges[source][v])  )
              {
                     weight[v] = weight[source] + edges[source][v];
                     path[v] = ( (GraphNode) items.elementAt(source) );
              }

            //controls the next element to be processed
            if (  !(getVertex(v).isMarked()) && weight[v] < smallestWeight  )
                {  
                   smallestWeight = weight[v];
		   next = v;
                }                          
         }//end for loop
 
        if(source == next) //if we've reached the end
         { break; }
      }//end while


       //this is where we work backwards using our second search key
       GraphNode start = new GraphNode( (String) searchKey2); //start from end
       GraphNode end = new GraphNode( (String) searchKey1);   //end at start (becasue we're using a stack
       returned.push(getVertex(start.toString()));  //before we do anything, push the first one onto stack


       if( weight[items.indexOf(start)] == Double.POSITIVE_INFINITY )
         { throw new GraphException("No Route To Host"); }
       else
         { distance = weight[items.indexOf(start)];  }

       while( ! start.equals(end) )       //push everything on the stack
       {
          int k = items.indexOf(start);
          returned.push(path[k]);
          start = path[k];
       }
       while(! returned.empty() )         //pop everything off the stack
         { retval.add(returned.pop()); }

      return retval;
      
   }
    

   /**
    * Unmarks item/vertex/GraphNode (whatever you want to call it).<br/>
    * <strong>Preconditions:</strong><br/>
    * <ul>
    *    <li>none</li>
    * </ul>
    * <strong>Postconditions:</strong><br/>
    * <ul>
    *    <li>Item's marked variable set to false</li>
    * </ul>
    */
   private void clearAllMarks()
   {
     for(int x = 0; x < items.size(); x++)
     {
        ( (GraphNode) items.elementAt(x) ).unmark();
     }
   }

   /**
    *fills a matrix with Infinity for all elements.<br/>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>Must be passed a matrix of doubles
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>All entries of the matrix will be equal to <code>Double.POSITIVE_INFINITY</code>
    *</UL>
    */
   private static void fillEmpty(double[][] matrix)
   {
       for(int i=0;i < matrix.length; i++) //first loop for each column
       {
          for(int j=0;j < matrix[i].length; j++) //second loop for each row
          {
             matrix[i][j] = Double.POSITIVE_INFINITY; //sets the primative double to the hex value of Infinity
          }
       }
   }

   /**
    *resized a 2D matrix of doubles.<br>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>Must be passed a matrix of doubles</LI>
    *  <LI>Matrix must be square (i.e. the length of each row and each column must be equal)</LI>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>Matrix will be resized</LI>
    *  <LI>growing matrixes will be padded with <code>Double.POSITIVE_INFINITY</code></LI>
    *  <LI>extra elements in shrinking matrixes will simply be discarded</LI>
    *  <LI>new matrix will be returned</LI>
    *</UL>
    */
   private static double[][] resize(double[][] matrix, int n)
   {
      double[][] temp = new double[n][n]; //creates the new matrix
      fillEmpty(temp);                  //fills it with POSITIVE_INFINITY
      //the loops to transfer elements
      for(int i=0;i < matrix.length; i++) //we use the length of n so extra entries are discarded. Column copy loop.
      {
         for(int j=0; j < matrix.length; j++) //row loop, once again using n for the same reason as stated above.
         {
            temp[i][j] = matrix[i][j]; //copy elements
         }
      }
      return temp; //reset the matrix reference to the new array we created. Old matrix will be garbage collected.
   }

   /**
    *deletes an element from a matrix and shifts all other elements.<br>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>Must pass it a valid index starting at 0</LI>
    *  <LI>Matrix must be square</LI>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>Will move all elements to the right and down of deleted items left and up respectivly</LI>
    *  <LI>Last row and column of matrix will be filled with <code>Double.POSITIVE_INFINITY</code></LI>
    *  <LI>Matrix size will <b>not</b> change</LI>
    *  <LI>Throws <code>GraphException</code> with explination (including index number) for invlaid index</LI>
    *</UL>
    */
   private static void delElement(double[][] matrix, int n)
   {

      //THREE STEPS
      //1. Clear Row/Column with offending data
      //2. Move everything back to new positions (Don't Drop Off Array!!!)
      //3. delete end row/column

      if(n > matrix.length) //checks to make sure the index is valid
       { throw new GraphException("Index of removal: " + n + " is out of bounds of this matrix moron!"); } //exception thrown for invalid index

      //clear date
      for(int i = 0; i < matrix.length; i++) //loop for clearing row
      {
          matrix[n][i] = Double.POSITIVE_INFINITY;
      }
      for(int j = 0; j < matrix.length; j++) //loop for clearing column
      {
          matrix[j][n] = Double.POSITIVE_INFINITY;
      }

      //now the hard part, filling in the gaps
        for(int x = 0; x < matrix.length - 1; x++) //starting with a row, -1 to keep from dropping off the end of the array
        {
          for(int y = 0; y < matrix.length -1 ; y++)//going through each y in the row, again -1 to keep from droping off array
          {
            if( x >= n && y >= n)
       { matrix[x][y] = matrix[x+1][y+1]; }//move item one up and one left to position
            else if( x>= n && y < n )
             { matrix[x][y] = matrix[x+1][y]; } //more stuff left one
            else if( y >= n && x < n)
             { matrix[x][y] = matrix[x][y+1] ; }//move stuff up one
          }
        }

     //now to delete that last row/column
     int A = matrix.length -1; //constant
     for(int b = matrix.length -1; b >= 0; b--) //b is arbitary and only used as a counter
     {
        matrix[b][A] = Double.POSITIVE_INFINITY; //column eraser
        matrix[A][b] = Double.POSITIVE_INFINITY; //row eraser
        //NOTE: the first time loop is run, we'll be resetting the right corner twice. No biggie
     }

   }

}//end class Graph
