/**
  *A node for a graph.
  *This class stores the data for the node
  *of a graph.
  */

import java.awt.Graphics;
import java.awt.Color;
import javax.swing.SwingUtilities;
import java.io.Serializable;

public class GraphNode extends KeyedItem implements Serializable 
{

  /**
    *location of the node's x coordinate.
    */
  protected int x;
  /**
    *location of the node's y coordinate.
    */
  protected int y;

  /**
    *indicates if the node is marked.
    */
  private boolean marked = false;

  /**
    *indicates node color.
    *If set to null, the default color passed to this node will be used during paint.
    */
  private Color color;

  /**
    *indicates node text color.
    *If set to null, the default color passed to this node will be used during paint.
    */
  private Color s_color;

  /**
    *indicates radius of node.
    *If set to null, the default radius passed to this node will be used.
    */
  private int radius;

  /**
    *creates a GraphNode.
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>Requires a string to be passed to it.
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>Creates a graph node from the string.</LI>
    *  <LI>Sets the X and Y fields to null (0)</LI>
    *</UL>
    */
    public GraphNode(String s)
    {  
       super(s); //initalize keyedItem
       this.x = 0; //set (x,y) to zero (null)
       this.y = 0; 
    }

  /**
    *creates a GraphNode with (x,y) coordinates.<br>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>Requires a string to be passed to it.</LI>
    *  <LI>It's x,y coordinates must be passed to it.</LI>
    *  <LI>(x,y) must be greater than zero</LI>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>Creates a graph node from the string.</LI>
    *  <LI>Sets the (x,y) coordinates of the node</LI>
    *  <LI>Throws an exception if x or y are less than zero</LI>
    *</UL>
    */
    public GraphNode(String s, int x, int y) throws GraphException
    {
       super(s);  //initalize KeyedItem with string
       if(x < 0 || y < 0) //check to see if (x,y) are positive
        { throw new GraphException("negative graphing coordinates"); }
       this.x = x; //establish these constants
       this.y = y;
    }
    
  /**
    *draws a node to a <code>Graphics</code> object.<br>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>Requires a <code>Graphics</code> be passed it it.</LI>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>Creates a node in the <code>Graphics</code> context using it's X and Y int fields</LI>
    *</UL>
    */    
    public void drawNode(Graphics g, Color col, Color text, int radius)
    {
       if(color != null) //checks to see if Node has custom color
        { g.setColor(color); } 
       else             //if not, set to default passed to it
        { g.setColor(col); }
 
        //checks to see if we should use the default radius
       int rad;
       if(this.radius <= 0) //if my radius is set to null
        { rad = radius; } //then use the radius passed to me
       else 
        { rad = this.radius; } //else use my preset radius

        //since X and Y are the center and the first two args are corners
        //for inscribing a circle in an invisible box, we simply
        //subtract the radius to the X and Y to form a vector which yeilds
        //top left corner. Wow, I actually learned something from Physics
        //before I dropped it!
        g.fillOval( x - rad, y - rad, 2*rad, 2*rad);

        //final check for default text color
         Color txt;
        if(s_color == null)
         { txt = text; }
        else
         { txt = s_color; } 

        g.setColor(txt); //sets the color for the text
           //calculates position of text so it's centered
           //position of new x = old x - (fontwidth  / 2)
           //same concept for y 
           int fontx = SwingUtilities.computeStringWidth(g.getFontMetrics(),this.toString());
            fontx = x - (fontx /2);
           int fonty = g.getFontMetrics().getHeight();
            fonty = y + (fonty / 2); //add to y..remember, Y IS TOP LEFT (this threw me off)
        g.drawString(this.toString(),fontx,fonty); //draws the string and centers it
    }

  /**sets the color for this node.<br>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>Requires a color
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>Node will no longer be painted with default color</LI>
    *  <LI>Node will be painted with <code>Color</code> <I>c</I></LI>
    *</UL>
    */
    public void setColor(Color c)
    { color = c; }

  /**
    *gets the color for this node.<br>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI><I>none</I>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>The current color of the node will be returned</LI>
    *  <LI>null will be returned if the node is using the default color</LI>
    *</UL>
    */
    public Color getColor()
    { return color; }

  /**sets the color for the text.<br>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI>Requires a color
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>text will no longer be painted with default color</LI>
    *  <LI>text will be painted with <code>Color</code> <I>c</I></LI>
    *</UL>
    */
    public void setTextColor(Color c)
    { s_color = c; }

  /**
    *gets the color for this node's text.<br>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI><I>none</I>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>The current color of the text will be returned</LI>
    *  <LI>null will be returned if the node is using the default color for its text</LI>
    *</UL>
    */
    public Color getTextColor()
    { return s_color; }
  /**
    *marks the node.<br>
    *Useful for graph searching routines.
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI><I>none</I>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>Node is marked.
    *</UL>
    */
    public void mark()
    { marked = true; }
  /**
    *unmarks the node.<br>
    *Useful for graph searching routines.
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI><I>none</I>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>Node is unmarked.
    *</UL>
    */
    public void unmark()
    { marked = false; }
  /**
    *returns the marked status of the node.<br>
    *Useful for graph searching routines.
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI><I>none</I>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>Returns weather the node is marked or unmarked.</LI>
    *  <LI>Node is unmarked by default.</LI>
    *</UL>
    */
    public boolean isMarked()
    { return marked; } 

  /**
    *returns the radius of the node.<br>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI><I>none</I>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>Returns the radius of the node.
    *</UL>
    */
    public int getRadius()
    { return radius; }

  /**
    *sets the radius of the node.<br>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI><I>none</I>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>Sets the radius of the node.</LI>
    *  <LI>A radius less than or equal to zero is considered a null and the default radius will be used</LI>
    *</UL>
    */
    public void setRadius(int r)
    { radius = r;}

  /**
    *sets the X coridinate of the node.<br>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI><I>none</I>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>Sets the X cord of the node.</LI>
    *</UL>
    */
    public void setX(int x)
    { this.x = x; }

  /**
    *sets the Y coridinate of the node.<br>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI><I>none</I>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>Sets the Y cord of the node.</LI>
    *</UL>
    */
    public void setY(int y)
    { this.y = y; }

  /**
    *sets the X and Y coridinates of the node.<br>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI><I>none</I>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>Sets the X and Y cords of the node.</LI>
    *</UL>
    */
    public void setXY(int x, int y)
    { this.y = y; this.x = x; }

  /**
    *gets the X coridinate of the node.<br>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI><I>none</I>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>Retruns the X cord of the Node.</LI>
    *</UL>
    */
    public int getX()
    { return x; }
  /**
    *gets the Y coridinate of the node.<br>
    *<B>Preconditions</B><BR>
    *<UL>
    *  <LI><I>none</I>
    *</UL><BR>
    *<B>PostConditions</B><BR>
    *<UL>
    *  <LI>Retruns the Y cord of the Node.</LI>
    *</UL>
    */
    public int getY()
    { return y; }

    //this is an overridden function from java.lang.Object
    //so to make the Vector check for duplicate items correctly
    //The javadoc comments from original function should carry to this one
    public boolean equals(Object o)
    {
      try
      {
        return ( (GraphNode) o ).toString().equals(this.toString());
      }
      catch (ClassCastException e)
      {
        return false;
      }
    }
}
